import { storiesOf } from '@storybook/react';
import React, { useEffect } from 'react';
import { Provider, useWallet, useWriteContract, NETWORKS, usePoller } from '..';
import { Button, Input, Divider, VStack } from '@chakra-ui/react';
import { ethers } from 'ethers';
import { useReadOnlyContract } from '../hooks/useReadOnlyContract';

const ADDRESS = '0x7e1D33FcF1C6b6fd301e0B7305dD40E543CF7135'; // Rinkeby
const ABI = [
  {
    inputs: [
      {
        internalType: 'string',
        name: '_greeting',
        type: 'string',
      },
    ],
    stateMutability: 'nonpayable',
    type: 'constructor',
  },
  {
    inputs: [],
    name: 'greet',
    outputs: [
      {
        internalType: 'string',
        name: '',
        type: 'string',
      },
    ],
    stateMutability: 'view',
    type: 'function',
  },
  {
    inputs: [
      {
        internalType: 'string',
        name: '_greeting',
        type: 'string',
      },
    ],
    name: 'setGreeting',
    outputs: [],
    stateMutability: 'nonpayable',
    type: 'function',
  },
  {
    inputs: [
      {
        internalType: 'address payable',
        name: '_to',
        type: 'address',
      },
    ],
    name: 'transferTo',
    outputs: [],
    stateMutability: 'payable',
    type: 'function',
  },
];

const Default = () => {
  const { connectWallet, disconnectWallet, connected } = useWallet();
  const [contract, isReady] = useWriteContract(ADDRESS, ABI);
  const [state, setState] = React.useState({
    newGreeting: '',
    toAddress: '',
    amount: '0',
  });

  usePoller(async () => {
    try {
      const greeting = await contract?.greet();
      console.log(greeting); // logs the greeting every second
    } catch (error) {
      console.log(error);
    }
  }, 1000);

  const handleGreet = async () => alert(await contract?.greet());
  const handleChangeState =
    (stateName: string) =>
    ({ target: { value } }) => {
      setState({ ...state, [stateName]: value });
    };
  const handleSetGreeting = async () => {
    await contract?.setGreeting(state.newGreeting);
    setState({ ...state, newGreeting: '' });
  };
  const handleTransferTo = async () => {
    await contract?.transferTo(state.toAddress, {
      value: ethers.utils.parseEther(state.amount),
    });
    setState({ ...state, toAddress: '', amount: '0' });
  };

  if (connected) {
    return (
      <VStack>
        <Button onClick={disconnectWallet}>Disconnect wallet</Button>
        <h3>Contract Methods</h3>
        <Button onClick={handleGreet}>greet</Button>
        <Divider />
        <Button
          disabled={!state.newGreeting || !isReady}
          onClick={handleSetGreeting}
        >
          setGreeting
        </Button>
        <Input
          value={state.newGreeting}
          placeholder="New Greeting!"
          onChange={handleChangeState('newGreeting')}
        />
        <Divider />
        <Button
          disabled={!(state.toAddress && state.amount)}
          onClick={handleTransferTo}
        >
          transferTo
        </Button>
        <Input
          value={state.toAddress}
          placeholder="0xjA123...."
          onChange={handleChangeState('toAddress')}
        />
        <Input
          placeholder="0.2"
          value={state.amount}
          onChange={handleChangeState('amount')}
        />
      </VStack>
    );
  }

  return (
    <>
      <Button onClick={connectWallet}>Connect Wallet</Button>
    </>
  );
};

storiesOf('Hooks/useWriteContract', module).add('Default', () => (
  <Provider network={NETWORKS.rinkeby} cacheProvider>
    <Default />
  </Provider>
));

const ReadContract = () => {
  const [contract, isReady] = useReadOnlyContract(ADDRESS, ABI);
  const [greeting, setGreeting] = React.useState('');

  useEffect(() => {
    async function exec() {
      setGreeting(await contract.greet());
    }
    if (isReady) {
      exec();
    }
  }, [contract, isReady]);

  return (
    <VStack>
      <h3>Greeting: {greeting}</h3>
    </VStack>
  );
};

storiesOf('Hooks/useReadContract', module).add('Default', () => (
  <Provider
    network={NETWORKS.rinkeby}
    rpcUrl="https://rinkeby.infura.io/v3/21bc321f21a54c528dc084f5ed7f8df7"
  >
    <ReadContract />
  </Provider>
));
